# pylint: disable=invalid-name, duplicate-code, no-name-in-module, broad-except, line-too-long
r"""Run deepl-fastapi-pw.

Test from the command line:
```bash

curl -XPOST http://127.0.0.1:8001/text/ -d"{\\"text\\": \\"this is a test\\", \\"to_lang\\": \\"zh\\"}"

curl "http://127.0.0.1:8001/text/?q=test%20this&to_lang=zh"
```
"""
import asyncio
import os
from signal import SIG_DFL, SIGINT, signal
from textwrap import dedent
from typing import Optional

# import nest_asyncio
# import portalocker
import uvicorn
from fastapi import FastAPI, Query

# import logzero
from logzero import logger
from pydantic import BaseModel

from deepl_fastapi_pw import __version__

# from deepl_scraper_pw.deepl_tr import deepl_tr
# from get_pwbrowser_sync.get_pwbrowser_sync import get_pwbrowser_sync
from deepl_fastapi_pw.deepl_tr import deepl_tr
from deepl_fastapi_pw.get_pwbrowser import get_pwbrowser

arun = asyncio.get_event_loop().run_until_complete

# lazy loading LOOP, wait for run_uvicorn to start first
# import lazy_import
# get_ppbrowser = lazy_import.lazy_module(get_ppbrowser)

port = 8001
# nest_asyncio.apply()  # need this for the whole thing to work


async def get_page():
    """Get a page."""
    try:
        # browser = get_pwbrowser_sync()
        browser = await get_pwbrowser()
    except Exception as exc:
        logger.error(exc)
        raise
    try:
        page = await browser.new_page()
    except Exception as exc:
        logger.error(exc)
        raise

    url = r"https://www.deepl.com/translator"
    try:
        await page.goto(url, timeout=16 * 1000)
    except Exception as exc:
        logger.error(exc)
        raise

    return page


LOOP = asyncio.get_event_loop()
PAGE = LOOP.run_until_complete(get_page())

descr = f"""curl -XPOST http://127.0.0.1:{port}/text/ -d '\u007b"text": "this is a test", "to_lang": "zh" \u007d'"""


class Text(BaseModel):  # pylint: disable=too-few-public-methods
    """Define Text model."""

    text: str
    from_lang: Optional[str] = None
    to_lang: Optional[str] = None
    description: Optional[str] = None


app = FastAPI(
    title="deepl-fastapi-pw",
    version=__version__,
    description=__doc__,
)


@app.post("/text/")
def post_text(q: Text):
    """Post to /text/.

    Post -d '\u007b"text": "this is a test", "to_lang": "zh" \u007d'
    to http://127.0.0.1:{port}/text/
    """
    text = q.text
    to_lang = q.to_lang
    from_lang = q.from_lang
    logger.debug("text: %s", text)

    # _ = sent_corr(text1, text2)
    try:
        # _ = await deepl_tr(
        _ = arun(
            deepl_tr(
                text,
                from_lang,
                to_lang,
                page=PAGE,
                headless=False,
            )
        )
    except Exception as exc:
        logger.exception(exc)
        _ = {"error": True, "message": str(exc)}

    return {"q": q, "result": _}


@app.get("/text/")
# async def get_text(
# q: Optional[str] = Query(
def get_text(
    q: str = Query(
        # None,
        "",
        max_length=5000,
        min_length=1,
        title="text to translate",
        description=(
            dedent(
                """
            paragraphs will be preserved.
            """.strip()
            )
        ),
    ),
    from_lang: Optional[str] = None,
    to_lang: Optional[str] = "zh",
):
    r"""Get text.

    curl "http://127.0.0.1:8001/text/?q=test%20this&to_lang=zh"

    Does not seem work, 'playwright\_impl\_sync_base.py... cannot switch to a different thread' TODO
    """
    result = {
        "q": q,
        "from_lang": from_lang,
        "to_lang": to_lang,
    }
    try:
        # trtext = await deepl_tr(
        trtext = arun(
            deepl_tr(
                q,
                from_lang,
                to_lang,
                page=PAGE,
            )
        )
    except Exception as exc:
        logger.exception(exc)
        trtext = str(exc)

    result.update({"trtext": trtext})
    result.update({"translation": trtext})

    logger.debug("result: %s", result)

    return result


def run_uvicorn():
    """Run uvicor.

    Must be run from a different file, e.g., run_uvicorn.py
    """
    uvicorn.run(
        # app="deepl_fastapi.deepl_server:app",
        app=app,  # this should work with python -m deepl_fastapi.deepl_server_async
        # host="0.0.0.0",
        port=port,
        # debug=True,
        # reload=True,
        # workers=2,
        # loop="asyncio",  # default "auto"
        # loop="uvloop",  # posix (linux and mac) only
    )


def main():
    """Start run_uvicorn."""
    logger.info("pid: %s", os.getpid())

    signal(SIGINT, SIG_DFL)
    print("ctrl-C to interrupt, visit http://...:../docs for api docs")

    run_uvicorn()


if __name__ == "__main__":
    main()

    # uvicorn.run(app, host="0.0.0.0", port=8000)
    # uvicorn.run("app.app:app",host='0.0.0.0', port=4557, reload=True, debug=True, workers=3)

    # uvicorn deepl_fastapi.deepl_server:app --reload
    # works with nest_asyncio

    _ = """
    deepl_scraper_pw.deel_tr is sync

    python -c "from deepl_fastapi_pw import deepl_server; import nest_asyncio; nest_asyncio.apply(); deepl_server.run_uvicorn()"
    python -c "from deepl_fastapi_pw import deepl_server; deepl_server.run_uvicorn()"

    greenlet.error: cannot switch to a different thread
    # import nest_asyncio
    # import sys
    """



r"""Instantiate a playwright chrominium browser.

Respect PWBROWSER_ environ variables in .env

Modified from mypython\get_pwbrowser.py

REPL：
from deepl_fastapi_pw.deepl_tr import deepl_tr
from deepl_fastapi_pw.get_pwbrowser import get_pwbrowser
from deepl_fastapi_pw.deepl_server_async import get_page

import asyncio
import atexit
from pathlib import Path
from typing import Optional, Union

import logzero
from logzero import logger
from playwright.async_api import Browser, async_playwright

# from playwright.sync_api import sync_playwright, Browser
# from get_pwbrowser.config import Settings
from pydantic import AnyUrl, BaseSettings  # pylint: disable=no-name-in-module

arun = asyncio.get_event_loop().run_until_complete
playwright = arun(async_playwright().start())

browser = arun(get_pwbrowser())

# page = arun(browser.new_page())  # works
page = arun(get_page())  # also works

res = arun(deepl_tr("this test", page=page))

"""
# pylint: disable=line-too-long
import asyncio
import atexit
import sys
from pathlib import Path
from typing import Optional, Union

import logzero
from logzero import logger
from playwright.__main__ import main
from playwright.async_api import Browser, async_playwright

# from playwright.sync_api import sync_playwright, Browser
# from get_pwbrowser.config import Settings
from pydantic import AnyUrl, BaseSettings  # pylint: disable=no-name-in-module

# check and install chromium/firefox if necessary, 5.79s
_ = sys.argv[:]  # save
# sys.argv = ["", "install", "firefox"]
sys.argv = ["", "install", "chromium"]
try:
    main()
except SystemExit:
    ...
except Exception as exc:
    logger.error(exc)
    raise
finally:  # restore
    sys.argv = _[:]


class Settings(BaseSettings):  # pylint: disable=too-few-public-methods
    """Configure params DEBUG HEADFUL PROXY.

    import asyncio
    from playwright.async_api import async_playwright

    async def main():
        playwright = await async_playwright().start()
        browser = await playwright.chromium.launch()

        # browser = await playwright.chromium.launch(headless=False)

        # Your code here

        # page = await browser.new_page()
        # await page.goto("https://www.google.com")

        # await browser.close()

    asyncio.run(main())

    """

    debug: bool = False
    headful: bool = False
    proxy: Optional[AnyUrl] = None

    class Config:  # pylint: disable=too-few-public-methods
        """Config."""

        env_prefix = "PWBROWSER_"
        # extra = "allow"
        env_file = ".env"
        env_file_encoding = "utf-8"  # pydantic doc

        logger.info(
            "env_prefix: %s, env_file: %s", env_prefix, Path(env_file).absolute()
        )


arun = asyncio.get_event_loop().run_until_complete
playwright = arun(async_playwright().start())


@atexit.register
def shutdown():
    """Stop playwright."""
    # pyright type error
    # error: Argument of type "None" cannot be assigned to parameter "future" of type "Awaitable[_T@run_until_complete]"
    # asyncio.get_event_loop().run_until_complete(playwright.stop())


config = Settings()
HEADLESS = not config.headful
DEBUG = config.debug
PROXY = config.proxy


# fmt: off
# def get_pwbrowser(
async def get_pwbrowser(
        headless: bool = HEADLESS,
        verbose: Union[bool, int] = DEBUG,
        proxy: Optional[Union[str, dict]] = PROXY,
        **kwargs
) -> Browser:
    # fmt: on
    """Instantiate a playwright chrominium browser (async)."""
    if isinstance(verbose, bool):
        verbose = 10 if verbose else 20
    logzero.loglevel(verbose)

    kwargs.update({
        "headless": headless,
    })

    if proxy:
        proxy = {"server": proxy}
        kwargs.update({
            "proxy": proxy,
        })

    _ = """
    try:
        # playwright = sync_playwright().start()
        playwright = await async_playwright().start()
    except Exception as exc:
        logger.error(exc)
        raise
    # """

    try:
        browser = await playwright.chromium.launch(**kwargs)
        # browser = playwright.chromium.launch(headless=False)
    except Exception as exc:
        logger.error(exc)
        raise

    return browser



"""Init."""
import nest_asyncio

# need this to rid of loop is already running
# get_page (runs pyppeteeer after main loop in uvicorn already started) after
# import nest_asyncio

# from .deepl_server import app  # make uvicorn deepl_fastapi:app possible

nest_asyncio.apply()

__version__ = "0.1.0a1"
# __all__ = ("app",)



"""Run uvicorn with deepl_fastapi.deepl_server:app.

uvicorn deepl_fastapi.deepl_server:app --reload
"""
# pylint: disable=duplicate-code
from pathlib import Path
from signal import SIG_DFL, SIGINT, signal

import portalocker
import uvicorn
from logzero import logger


def run_uvicorn(host="127.0.0.1", port=8000, debug=False, reload=False):
    """Start uvicorn."""
    uvicorn.run(
        # app="deepl_fastapi.deepl_server:app",
        app="deepl_fastapi_pw.deepl_server:app",
        host=host,
        port=port,
        debug=debug,
        reload=reload,
        # workers=2,
        # loop="asyncio",  # default "auto"
        # loop="uvloop",  # posix (linux and mac) only
    )


def main():
    """Run main."""
    signal(SIGINT, SIG_DFL)
    print("ctrl-C to interrupt")

    file_ = Path(__file__).parent / "deepl_server.py"
    lockfile = Path(f"{file_}.portalocker.lock")
    if not Path(lockfile).exists():
        Path(lockfile).touch()
    try:
        with open(lockfile, "r+", encoding="utf8") as file:
            # portalocker.lock(file, portalocker.constants.LOCK_EX)
            portalocker.lock(file, portalocker.LOCK_EX | portalocker.LOCK_NB)
    except Exception as exc:
        logger.debug(exc)
        logger.error("Another copy is running, exiting...")
        raise SystemExit(1) from exc
        # raise
    finally:
        # LOOP.close()
        ...

    try:
        run_uvicorn()
    except Exception as exc:
        logger.error(exc)
        raise SystemExit(1) from exc


if __name__ == "__main__":
    main()



r"""Scrape deepl via playwright (get_pwbrowser_sync).

org deepl_tr_pp

import os
from pathlib import Path
os.environ['PYTHONPATH'] = Path(r"../get-pwbrowser-sync")

Modi from pypi deepl-scraper-playwright\deepl_scraper_pw\deepl_tr.py
"""
# pylint: disable=line-too-long, too-many-arguments, too-many-locals, too-many-branches, too-many-statements
import os
import re
import sys
from random import randint
from time import sleep
from typing import Any, Callable, Optional, Union
from urllib.parse import quote

import logzero
from about_time import about_time
from logzero import logger
from pyquery import PyQuery as pq

# async version, necessary since sync version wont run in uvicorn/fastapi
from deepl_fastapi_pw.get_pwbrowser import get_pwbrowser  # , loop  async version

# from get_pwbrowser_sync import get_pwbrowser_sync as get_pwbrowser, loop


URL = r"https://www.deepl.com/translator"


def with_func_attrs(**attrs: Any) -> Callable:
    """with_func_attrs"""

    def with_attrs(fct: Callable) -> Callable:
        for key, val in attrs.items():
            setattr(fct, key, val)
        return fct

    return with_attrs


@with_func_attrs(from_lang="", to_lang="", text="")
async def deepl_tr(
    text: str,
    from_lang: Optional[str] = "auto",
    to_lang: Optional[str] = "zh",
    page=None,
    verbose: Union[bool, int] = False,
    timeout: float = 5,
    headless: Optional[bool] = None,
):
    """Deepl via playwright-sync.

    text = "Test it and\n\n more"
    from_lang="auto"
    page=None
    to_lang="zh"
    verbose=True
    """
    if from_lang is None:
        from_lang = "auto"
    if to_lang is None:
        to_lang = "zh"

    # set verbose=40 to turn most things off
    if isinstance(verbose, bool):
        if verbose:
            logzero.setup_default_logger(level=10)
        else:
            logzero.setup_default_logger(level=20)
    else:  # integer: log_level
        logzero.setup_default_logger(level=verbose)

    if os.environ.get("DEBUG"):
        logzero.loglevel(10)

    logger.debug(" Entry ")

    try:
        text = text.strip()
    except Exception as exc:
        logger.error(exc)
        logger.info(" not a string?")
        raise

    # if text remains the same but from_lang or to_lang changed, attach random string \d_
    # if text.strip() == deepl_tr.strip() and (deepl_tr.from_lang
    logger.debug(
        "text==deepl_tr.text: %s==%s, from_lang==deepl_tr.from_lang: %s==%s, to_lang==deepl_tr.to_lang: %s==%s",
        text,
        deepl_tr.text,
        from_lang,
        deepl_tr.from_lang,
        to_lang,
        deepl_tr.to_lang,
    )

    same_langs = from_lang == deepl_tr.from_lang and to_lang == deepl_tr.to_lang

    if text == deepl_tr.text:
        if not same_langs:
            deepl_tr.text = text
            _ = randint(1, 1000)
            logger.debug("attaching extra")
            text = f"{text} {_}_"
    logger.debug("text: %s", text)

    deepl_tr.from_lang = from_lang
    deepl_tr.to_lang = to_lang

    # reuse page
    try:
        # deepl_tr.page  # run previously
        page = deepl_tr.page
    except AttributeError:
        if page is None:
            try:
                if headless is None:
                    browser = await get_pwbrowser()
                else:
                    browser = await get_pwbrowser(headless=headless)
            except Exception as exc:
                logger.error(exc)
                raise

            try:
                page = await browser.new_page()
            except Exception as exc:
                logger.error(exc)
                raise

            url = r"https://www.deepl.com/translator"
            try:
                await page.goto(url, timeout=45 * 1000)
            except Exception as exc:
                logger.error(exc)
                raise

            deepl_tr.page = page
        else:
            deepl_tr.page = page

    page = deepl_tr.page

    url0 = f"{URL}#{from_lang}/{to_lang}/"

    url_ = f"{URL}#{from_lang}/{to_lang}/{quote(text)}"

    # selector = ".lmt__language_select--target > button > span"

    if verbose < 11 or verbose is True:
        _ = False  # silent
    else:
        _ = True

    content = await page.content()

    # with CodeTimer(name="fetching", unit="s", silent=_):
    with about_time() as dur:
        try:
            content = await page.content()
        except Exception as exc:
            logger.error(exc)
            raise

        doc = pq(content)
        text_old = doc("#source-dummydiv").html()
        logger.debug("Old source: %s", text_old)

        try:
            deepl_tr.first_run
        except AttributeError:
            deepl_tr.first_run = 1
            text_old = "_some unlikely random text_"

        logger.debug(
            "text.strip(): %s, text_old.strip(): %s", text.strip(), text_old.strip()
        )

        # selector = "div.lmt__translations_as_text"
        if text.strip() == text_old.strip() and same_langs:
            logger.debug(" ** early result: ** ")
            logger.debug(
                "%s, %s", text, doc(".lmt__translations_as_text__text_btn").html()
            )
            doc = pq(await page.content())
            # content = doc(".lmt__translations_as_text__text_btn").text()
            content = doc(".lmt__translations_as_text__text_btn").html()
        else:
            # record content
            try:
                # page.goto(url_)
                await page.goto(url0)
            except Exception as exc:
                logger.error(exc)
                raise

            try:
                # page.wait_for_selector(".lmt__translations_as_text", timeout=20000)
                await page.wait_for_selector(".lmt__target_textarea", timeout=20000)
            except Exception as exc:
                logger.error(exc)
                raise

            doc = pq(await page.content())
            # content_old = doc(".lmt__translations_as_text__text_btn").text()
            content_old = doc(".lmt__translations_as_text__text_btn").html()

            # selector = ".lmt__translations_as_text"
            # selector = ".lmt__textarea.lmt__target_textarea.lmt__textarea_base_style"
            # selector = ".lmt__textarea.lmt__target_textarea"
            # selector = '.lmt__translations_as_text__text_btn'
            try:
                await page.goto(url_)
            except Exception as exc:
                logger.error(exc)
                raise

            try:
                # page.wait_for_selector(".lmt__translations_as_text", timeout=20000)
                await page.wait_for_selector(".lmt__target_textarea", timeout=20000)
            except Exception as exc:
                logger.error(exc)
                raise

            doc = pq(await page.content())
            content = doc(".lmt__translations_as_text__text_btn").text()

            logger.debug("content_old: [%s], \n\t content: [%s]", content_old, content)

            # loop until content changed
            idx = 0
            # bound = 50  # 5s
            while idx < timeout / 0.1:
                idx += 1
                sleep(0.1)
                doc = pq(await page.content())
                content = doc(".lmt__translations_as_text__text_btn").html()
                logger.debug(
                    "content_old: (%s), \n\tcontent: (%s)", content_old, content
                )

                if content_old != content and bool(content):
                    break

            logger.debug(" loop: %s", idx)

    # deepl_tr.dur = dur
    deepl_tr.dur = dur.duration_human

    logger.debug(" Fini ")

    # remove possible attached suffix
    content = re.sub(r"[\d]+_$", "", content.strip()).strip()

    return content


def main():
    """Main."""
    text = "test this and that and more"
    res = deepl_tr(text)
    logger.info("%s, %s, time elased: %s", text, res, deepl_tr.dur)

    res = deepl_tr(text, to_lang="de")

    logger.info("%s, %s, time elased: %s", text, res, deepl_tr.dur)

    # input("Press Enter to continue...")

    if len(sys.argv) > 1:
        text = " ".join(sys.argv[1:])
    else:
        text = "test this and that"

    res = deepl_tr(text)

    logger.info("%s, %s, time elased: %s", text, res, deepl_tr.dur)

    res = deepl_tr(text, to_lang="de")

    logger.info("%s, %s, time elased: %s", text, res, deepl_tr.dur)

    # input("Press Enter to continue...")

    # stop loop, will close all browsers
    # loop.stop()
    del deepl_tr.page


if __name__ == "__main__":
    main()
    _ = """
    try:
        main()
    except Exception as exc:
        logger.error(exc)
    # """



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['deepl_fastapi_pw']

package_data = \
{'': ['*']}

install_requires = \
['about-time>=4.2.1,<5.0.0',
 'fastapi>=0.63.0,<0.64.0',
 'logzero>=1.6.3,<2.0.0',
 'nest-asyncio>=1.5.6,<2.0.0',
 'playwright>=1.32.1,<2.0.0',
 'portalocker>=2.7.0,<3.0.0',
 'pyquery>=2.0.0,<3.0.0',
 'uvicorn>=0.13.4,<0.14.0']

entry_points = \
{'console_scripts': ['deepl-fastapi-pw = deepl_fastapi_pw.run_uvicorn:main']}

setup_kwargs = {
    'name': 'deepl-fastapi-pw',
    'version': '0.1.0a1',
    'description': 'deepl via fastapi using deepl-scraper-pw',
    'long_description': '# deepl-fastapi-pw\n<!--- repo-name  pypi-name  mod_name func_name --->\n[![tests](https://github.com/ffreemt/deepl-fastapi-playwright/actions/workflows/routine-tests.yml/badge.svg)][![python](https://img.shields.io/static/v1?label=python+&message=3.7%2B&color=blue)](https://img.shields.io/static/v1?label=python+&message=3.7%2B&color=blue)[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)[![PyPI version](https://badge.fury.io/py/deepl-fastapi.svg)](https://badge.fury.io/py/deepl-fastapi-pw)\n\nYour own deepl server via fastapi and playwright, cross-platform (Windows/Linux/MacOs) with API for OmegaT\n\n## Installation\n*   Create a virual environment: optional but recommended\n    e.g.,\n    ```bash\n    # Linux and friends\n    python3.7 -m venv .venv\n    source .venv/bin/activate\n\n    # Windows\n    # py -3.7 -m venv .venv\n    # .venv\\Scripts\\activate\n    ```\n\n```bash\npip install deepl-fastapi-pw\n```\nor (if your use poetry)\n```bash\npoetry add deepl-fastapi-pw\n```\nor\n```\n pip install git+https://github.com/ffreemt/deepl-fastapi-playwright.git\n```\nor\n*   Clone the repo [https://github.com/ffreemt/deepl-fastapi-playwright.git](https://github.com/ffreemt/deepl-fastapi-playwrigh.git)\n    ```bash\n    git clone https://github.com/ffreemt/deepl-fastapi-playwright.git\n    ```\n    and `cd deepl-fastapi-playwright`\n*   `pip install -r requirements.txt\n    * or ``poetry install``\n\n## Usage\n\n*   Start the server\n\nUse uvicorn directly (note the `deepl_server` module, not `run_uvicorn`)\n```bash\nuvicorn deepl_fastapi_pw.deepl_server_async:app\n```\n\nor\n```bash\npython  -m deepl_fastapi_pw.deepl_server_async\n```\n\nor run the server on the external net, for example at port 9888\n```\nuvicorn deepl_fastapi_pw.deepl_server:app --reload --host 0.0.0.0 --port 9888\n```\n\n*   Explore and consume\n\nPoint your browser to [http://127.0.0.1:8001/text/?q=test&to_lang=zh](http://127.0.0.1:8000/text/?q=test&to_lang=zh)\n\nOr in python code (`pip install requests` first)\n```python\nimport requests\n\n# get\nurl =  "http://127.0.0.1:8001/text/?q=test me&to_lang=zh"\nprint(requests.get(url).json())\n# {\'q\': \'test me\', \'from_lang\': None, \'to_lang\': \'zh\',\n# \'trtext\': \'考我 试探我 测试我 试探\'}\n\n# post\ntext = "test this and that"\ndata = {"text": text, "to_lang": "zh"}\nresp = requests.post("http://127.0.0.1:8001/text", json=data)\nprint(resp.json())\n# {\'q\': {\'text\': \'test this and that\', \'from_lang\': None, \'to_lang\': \'zh\', \'description\': None},\n# \'result\': \'试探 左右逢源 检验 审时度势\'}\n\n```\n\n## Interactive Docs (Swagger UI)\n\n [http://127.0.0.1:8000/docs](http://127.0.0.1:8000/docs)\n',
    'author': 'freemt',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ffreemt/deepl-fastapi-playwright',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.3,<4.0.0',
}


setup(**setup_kwargs)
